package homegear;

import java.io.*;
import java.util.*;
import java.text.*;
import java.security.*;
import java.security.cert.*;
import java.net.*;
import javax.net.*;
import javax.net.ssl.*;

/**
  * Representation of a device parameter. 
  */
  
public class HomeGearParameter {
    private int id, channel;
    private String name;
    private HomeGearArray data;
/** * Peer name */
    public String peerName;

/**
  * Constructs a parameter. 
  * @param id_ ID of the per
  * @param channel_ channel of the parameter
  * @param name_ name of the parameter
  * @param peerName_ name of the peer. It can be null and is only used to produce readable output.
  * @param data_ parameter data. The data buffer is copied using {@link HomeGearArray.copy()}.
  */
    public HomeGearParameter(int id_, int channel_, String name_, String peerName_, HomeGearArray data_) {
	id = id_;
	channel = channel_;
	name = name_;
	peerName = peerName_;
	data = data_ != null ? data_.copy() : null;
    }

/**
  * Returns peer id
  * @return peer id.
  */
    public int id() {
	return id;
    }

/**
  * Returns channel
  * @return channel.
  */
    public int channel() {
	return channel;
    }

/**
  * Returns parameter name
  * @return parameter name
  */
    public String name() {
	return name;
    }

/**
  * Returns parameter data
  * @return parameter data
  */
    public HomeGearArray data() {
	return data;
    }

/**
  * Returns the value as a String or null if value can't be determined.
  * @param data_ the parameter data.
  * @return the value as a String
  */
    public static String stringValue( HomeGearArray data_ ) {
	if ( data_==null ) return null;
	try {
	    // obtain value list
	    HomeGearArray valueList = null;
	    int i = data_.find("TYPE");
	    if ( (i>0) && data_.stringValue("",i).equals("ENUM") ) {
		try {
		    i = data_.find("VALUE_LIST");
		    if ( i>0 ) valueList = data_.child(i);
		}
		catch ( Exception e ) {
		    valueList = null;
		}
	    }
            // get value
	    i = data_.find("VALUE");
	    if ( i<0 ) return null;
	    if ( valueList != null ) {
		i=data_.intValue(i);
		return valueList.stringValue("\n	", i);
	    }
	    return data_.stringValue("\n	", i);
	} catch ( Exception e ) {
	}
	return null;
    }

/**
  * Returns the value as a String or null if value can't be determined
  * @return the value as a String
  */
    public String stringValue() {
	return stringValue(data);
    }

/**
  * Updates data.
  * Data is only updated if value has been changed.
  * @param data_ new data. The data buffer is copied using {@link HomeGearArray.copy()}.
  * @return true if data is changed
  */
    public Boolean update( HomeGearArray data_ ) {
	if ( data == null ) return false;
	String oldVal = stringValue();
	if ( (oldVal == null) || !oldVal.equals(stringValue(data_)) ) {
	    data = data_.copy();
	    return true;
	}
	return false;
    }

/**
  * Comapare with another instance 
  * @param a The other instance.
  * @return True if equal.
  */
    public boolean equals(Object a) {
	if ( ! (a instanceof HomeGearParameter) ) return false;
	return (((HomeGearParameter)a).id()==id) && (((HomeGearParameter)a).channel()==channel) && name.equals(((HomeGearParameter)a).name());
    }


/**
  * Returns a string representation.
  * @return a string representation.
  */
     public String toString() {
        String value = stringValue();
        return ( ( (peerName!=null) && (peerName.length()>0) ) ? peerName : "(" + id + ")" ) + ": " + name + " (" + id + ":" + channel + "): " + (value != null ? value : "(?)" );
     }
}
