package homegear;

import java.io.*;
import java.util.*;
import java.text.*;

/**
  * Represantation of response from the homegear server. 
  * Its a {@link HomeGearArray} with additional information about kind of response.
  * If the response is an array or a struct this variable is stored directly, otherwise a
  * array consisting in one variable is generated.
  * The response has to be created using the {@link #createResponse(boolean,byte[])} method.
  */

public class HomeGearResponse extends HomeGearArray {
    private boolean isError; 			// true is response is an error response
    
// private constructer because super has to be called first.
    private HomeGearResponse(boolean isError_, boolean struct, byte[] buf, int offs) throws IllegalArgumentException {
	super(struct, buf, offs);
	isError = isError_; 
    }
    
/**
  * Constructs an error response from raw data.
  * @param isError_ true if it is an error response.
  * @param buf Buffer containing raw response data.
  * @throws IllegalArgumentException if a parameter is invalid.
  */
    public static HomeGearResponse createResponse(boolean isError_, byte[] buf) throws IllegalArgumentException {
	int t = (buf[0]<<24) | ((buf[1] & 255)<<16) | ((buf[2] & 255)<<8) | (buf[3] & 255);
	if ( t == TYPE_ARRAY || t == TYPE_STRUCT ) 
	    return new HomeGearResponse(isError_, t == TYPE_STRUCT, buf, 4);

	byte[] buf2 = new byte[buf.length+4];
	System.arraycopy(new byte[] {0,0,0,1},0, buf2,0, 4);;
	System.arraycopy(buf,0, buf2,4, buf.length);
	return new HomeGearResponse(isError_, false, buf2, 0);
    }

    
/**
  * Return true if it is an error response.  
  * @return true if it is an error response.
  */
    public boolean isError() {
	return isError;
    }
}  

